@@ -30,7 +30,7 @@ from .constants import SPSS_MAX_ENCODING
 class SPSSFile(object):
     """Base class for opening and closing SPSS files"""
 
-    def __init__(self, spss_file, mode="rb", unicode=True, locale=None):
+    def __init__(self, spss_file: str, mode: str = "rb", unicode: bool = True, locale: str = None):
         if config.spssio_module is None:
             raise ValueError(
                 "Missing spssio module. Set location of module by changing pyspssio.config.spssio_module = path/to/module.ext"
@@ -64,20 +64,34 @@ class SPSSFile(object):
             self._load_libs(libs, loader)
             self.spssio = loader(config.spssio_module)
 
-        # functions for opening and closing
+        # functions for opening and closing (always open with utf-8 encoded filenames)
         self._modes = {
-            "rb": {"open": self.spssio.spssOpenRead, "close": self.spssio.spssCloseRead},
-            "wb": {"open": self.spssio.spssOpenWrite, "close": self.spssio.spssCloseWrite},
-            "ab": {"open": self.spssio.spssOpenAppend, "close": self.spssio.spssCloseAppend},
+            "rb": {"open": self.spssio.spssOpenReadU8, "close": self.spssio.spssCloseRead},
+            "wb": {"open": self.spssio.spssOpenWriteU8, "close": self.spssio.spssCloseWrite},
+            "ab": {"open": self.spssio.spssOpenAppendU8, "close": self.spssio.spssCloseAppend},
         }
 
         # initialize in unicode or codepage mode
         self.interface_encoding = unicode
 
-        self.system_locale = ".".join(lc.getlocale())
-        self.locale = self.set_locale(self.system_locale if not locale else locale)
+        # get current locale information and set initial encoding
+        language_code, encoding_category = lc.getlocale()
+        self.system_locale = ".".join((language_code, encoding_category))
+        self.encoding = "utf-8" if unicode else encoding_category
+        # test setting initial locale to obtain encoding
+        if locale:
+            lc.setlocale(lc.LC_ALL, locale)
+            language_code, encoding_category = lc.getlocale()
+            locale = ".".join((language_code, encoding_category))
+            self.encoding = "utf-8" if unicode else encoding_category
+            # reset locale after getting use locale information
+            lc.setlocale(lc.LC_ALL, self.system_locale)
 
-        # file handle and encoding
+        # set I/O locale and initial encoding
+        self.locale = self.set_locale(self.system_locale if not locale else locale)
+        # self.encoding = "utf-8" if unicode else encoding_category
+
+        # match I/O encoding based on file information for read/append modes
         if self.mode in ["rb", "ab"]:
             self.mode = "rb"
             self.fh = self.open()
@@ -91,7 +105,7 @@ class SPSSFile(object):
         self.encoding = self.file_encoding
 
         # test encoding compatibility
-        compatible = self.is_compatible_encoding()
+        compatible = self.is_compatible_encoding
         if not compatible:
             UnicodeWarning("File encoding may not be compatible with SPSS I/O interface encoding")
 
@@ -156,17 +170,17 @@ class SPSSFile(object):
         return sysmis.value
 
     @property
-    def interface_encoding(self):
-        """Get or set I/O interface mode
+    def interface_encoding(self) -> int:
+        """I/O interface mode (Unicode or code page)
 
-        0 = SPSS_ENCODING_CODEPAGE
-        1 = SPSS_ENCODING_UTF8
+        - 0 = SPSS_ENCODING_CODEPAGE
+        - 1 = SPSS_ENCODING_UTF8
         """
 
         return self.spssio.spssGetInterfaceEncoding()
 
     @interface_encoding.setter
-    def interface_encoding(self, unicode):
+    def interface_encoding(self, unicode: bool):
         func = self.spssio.spssSetInterfaceEncoding
         func.argtypes = [c_int]
         retcode = func(c_int(int(unicode)))
@@ -174,24 +188,24 @@ class SPSSFile(object):
         return
 
     @property
-    def file_encoding(self):
+    def file_encoding(self) -> str:
         """File encoding reported by I/O module"""
 
         func = self.spssio.spssGetFileEncoding
         psz_encoding = create_string_buffer(SPSS_MAX_ENCODING + 1)
         retcode = func(self.fh, psz_encoding)
         warn_or_raise(retcode, func)
-        return psz_encoding.value.decode("utf-8")
+        return psz_encoding.value.decode(self.encoding)
 
-    def set_locale(self, locale):
-        """Set I/O module with a specific locale"""
+    def set_locale(self, locale: str) -> str:
+        """Set I/O module to a specific locale"""
 
         func = self.spssio.spssSetLocale
         func.argtypes = [c_int, c_char_p]
         func.restype = c_char_p
-        result = func(lc.LC_ALL, locale.encode("utf-8"))
+        result = func(lc.LC_ALL, locale.encode(self.encoding))
         if result:
-            return result.decode("utf-8")
+            return result.decode(self.encoding)
         else:
             warnings.warn(
                 "Failed to set locale to: "
@@ -203,24 +217,44 @@ class SPSSFile(object):
             )
             return ".".join(lc.getlocale())
 
-    def is_compatible_encoding(self):
-        """Check encoding compatibility"""
+    @property
+    def is_compatible_encoding(self) -> bool:
+        """Check encoding compatibility
+
+        From I/O module documentation: "This function determines whether the file's encoding is compatible with the current interface encoding.
+        The result value ... will be false when reading a code page file in UTF-8 mode, when reading
+        a UTF-8 file in code page mode when reading a code page file encoded in other than the current locale's
+        code page, or when reading a file with numbers represented in reverse bit order. If the encoding is
+        incompatible, data stored in the file by other applications, particularly Data Entry for Windows, may be
+        unreliable."
+        """
 
         func = self.spssio.spssIsCompatibleEncoding
         func.argtypes = [c_int, POINTER(c_int)]
         b_compatible = c_int()
         retcode = func(self.fh, b_compatible)
         warn_or_raise(retcode, func)
-        return b_compatible.value
+        return bool(b_compatible.value)
 
-    def open(self):
-        """Open file"""
+    def open(self) -> int:
+        """Open file
+
+        Returns file handle that is used for most other I/O module functions.
+
+        Notes
+        -----
+        Filenames are always encoded in UTF-8 regardless of interface mode and locale settings.
+        This is to avoid issues where a filename uses special characters that aren't available
+        in the encoding defined by the file itself. For example, a Windows-1252 .sav file
+        which uses Chinese (or other special multibyte characters) in its filename.
+        """
 
         with open(self.filename, self.mode) as f:
             fh = c_int(f.fileno())
-        filename_adjusted = os.path.expanduser(os.path.abspath(self.filename)).encode("utf-8")
+        filename_adjusted = os.path.expanduser(os.path.abspath(self.filename))
+        filename_encoded = filename_adjusted.encode("utf-8")
         func = self._modes[self.mode]["open"]
-        retcode = func(filename_adjusted, byref(fh))
+        retcode = func(filename_encoded, byref(fh))
         warn_or_raise(retcode, func)
         return fh
 
@@ -232,12 +266,12 @@ class SPSSFile(object):
         warn_or_raise(retcode, func)
 
     @property
-    def compression(self):
-        """Get or set compression level
+    def compression(self) -> int:
+        """Compression level
 
-        0 = No compression
-        1 = SAV
-        2 = ZSAV
+        - 0 = No compression
+        - 1 = SAV
+        - 2 = ZSAV
         """
 
         func = self.spssio.spssGetCompression
@@ -254,8 +288,18 @@ class SPSSFile(object):
         warn_or_raise(retcode, func)
 
     @property
-    def release_info(self):
-        """Basic file information"""
+    def release_info(self) -> dict:
+        """Basic file information
+
+        - release number
+        - release subnumber
+        - fixpack number
+        - machine code
+        - floating-point representation code
+        - compression scheme code
+        - big/little-endian code
+        - character representation code
+        """
 
         fields = [
             "release number",
@@ -274,7 +318,7 @@ class SPSSFile(object):
         return dict([(item, rel_info_arr[i]) for i, item in enumerate(fields)])
 
     @property
-    def var_count(self):
+    def var_count(self) -> int:
         """Number of variables"""
 
         func = self.spssio.spssGetNumberofVariables
@@ -285,7 +329,7 @@ class SPSSFile(object):
         return num_vars.value
 
     @property
-    def case_count(self):
+    def case_count(self) -> int:
         """Number of cases"""
 
         func = self.spssio.spssGetNumberofCases
